# ALMA Data Reduction Script

# Notes:
# - Tsys for DV11 has a different spectral shape in Y polarization, spws 9,15,
#   where Tsys is higher at the center than edges of band

# DA50,DA64,DV07,DV14: large phase delay in bandpass was calibrated.

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.2.2', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.2.2')


# CALIBRATE_AMPLI: J0238+1636
# CALIBRATE_ATMOSPHERE: J0215-0222,J0238+1636,J0241-0815,mira
# CALIBRATE_BANDPASS: J0241-0815
# CALIBRATE_FLUX: J0238+1636
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0215-0222
# CALIBRATE_POINTING: J0241-0815
# OBSERVE_TARGET: mira

# Using reference antenna = DA51

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X916b15_X926.ms') == False:
    importasdm('uid___A002_X916b15_X926', asis='Antenna Station Receiver Source CalAtmosphere CalWVR')
  if applyonly != True: es.fixForCSV2555('uid___A002_X916b15_X926.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X916b15_X926.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.listobs')
  listobs(vis = 'uid___A002_X916b15_X926.ms',
    listfile = 'uid___A002_X916b15_X926.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X916b15_X926.ms',
    mode = 'manual',
    spw = '1~27',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X916b15_X926.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X916b15_X926.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X916b15_X926.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X916b15_X926.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.wvr') 
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X916b15_X926.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X916b15_X926.ms',
    caltable = 'uid___A002_X916b15_X926.ms.wvr',
    toffset = 0,
    tie = ['mira,J0215-0222'],
    statsource = 'mira')
  
  casalog.setlogfile(mylogfile)
  
  # This is a temporary workaround, which will be included in a future version of CASA
  
  tb.open('uid___A002_X916b15_X926.ms.wvr', nomodify=False)
  count = 0
  numrows = tb.nrows()
  mycparamcol = tb.getcol('CPARAM')
  for i in range(0, numrows):
      if mycparamcol[0][0][i] == (1.+0.j):
          tb.putcell('FLAG', i, [[True]])
          count += 1
  tb.close()
  del mycparamcol
  if(numrows>0):
      print 'Flagged', count, 'of', numrows, 'solutions =', 100.*count/float(numrows),'%'
  
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.wvr.smooth') 
  
  smoothcal(vis = 'uid___A002_X916b15_X926.ms',
    tablein = 'uid___A002_X916b15_X926.ms.wvr',
    caltable = 'uid___A002_X916b15_X926.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_X916b15_X926.ms.wvr.smooth', spw='11', antenna='DA51',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_X916b15_X926.ms.wvr.smooth.plots/uid___A002_X916b15_X926.ms.wvr.smooth') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.tsys') 
  gencal(vis = 'uid___A002_X916b15_X926.ms',
    caltable = 'uid___A002_X916b15_X926.ms.tsys',
    caltype = 'tsys')
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X916b15_X926.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
    field='', figfile='uid___A002_X916b15_X926.ms.tsys.plots.overlayTime/uid___A002_X916b15_X926.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.tsys', msName='uid___A002_X916b15_X926.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: the correction for antenna DV19 is larger than 2mm.
  
  # Position for antenna DV19 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DA64 is larger than 2mm.
  
  # Position for antenna DA64 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA49 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA48 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA61 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: no baseline run found for antenna DA45.
  
  # Position for antenna DV10 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DV13 is larger than 2mm.
  
  # Position for antenna DV13 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DA46 is larger than 2mm.
  
  # Position for antenna DA46 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DA41 is larger than 2mm.
  
  # Position for antenna DA41 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DV14 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA43 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DV15 is larger than 2mm.
  
  # Position for antenna DV15 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DA62 is larger than 2mm.
  
  # Position for antenna DA62 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna PM03 is larger than 2mm.
  
  # Position for antenna PM03 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna PM02.
  
  # Position for antenna DV08 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DV09 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DV11 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA50 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DA51 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DA57 is larger than 2mm.
  
  # Position for antenna DA57 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Position for antenna DA55 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DV07 is larger than 2mm.
  
  # Position for antenna DV07 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Position for antenna DV04 is derived from baseline run made on 2014-10-20 00:38:20.
  
  # Note: the correction for antenna DV01 is larger than 2mm.
  
  # Position for antenna DV01 is derived from baseline run made on 2014-09-05 03:49:41.
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.antpos') 
  gencal(vis = 'uid___A002_X916b15_X926.ms',
    caltable = 'uid___A002_X916b15_X926.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV19,DA64,DA49,DA48,DA61,DV11,DV10,DV13,DA46,DA41,DV14,DA43,DV15,DA62,PM03,DA55,DA50,DA51,DA57,DV08,DV09,DV07,DV04,DV01',
    parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
  #  parameter = [-0.00181662436924,0.00261098720807,0.00106047899574,-0.00216401536542,0.00150183599432,0.000644677550705,0.000428697211363,-0.000503982049201,-0.000230741836373,0.000578655879557,-0.00157797915241,-0.00070093449596,0.00026645229555,4.32652504668e-05,-0.00010274868514,-0.000599562958936,0.000707300722662,0.000477526906042,0.000310009289689,-0.000586255334413,-0.000347047436497,-0.00321740140646,0.00302617422003,0.00165461650533,-0.00136784547481,0.00197859798575,0.000133322045601,-0.000833209869407,0.00189550797013,0.00107475618105,0.000644528128892,-0.00143047081611,-0.00101184274239,-0.00128085004815,0.00101580639757,0.000885802188304,-0.00455063894171,0.00357201047151,-0.000393024698505,0.00150644285176,-0.0026544047851,-0.00122481638265,-0.00109307433206,0.00196194413934,0.000799193780456,7.93021172285e-07,-1.01141631603e-06,-7.34813511372e-07,0.000929844869988,-0.00121202373745,-0.000479471560804,0.000314686912932,-0.000782491788217,-0.000635081686242,-0.000978342723101,0.00163009297103,0.000727996230125,-0.000287301911653,0.000591817343895,-0.000230700765117,-0.000393744228921,0.000555729369593,-1.09247408639e-05,-0.00149839535031,0.00169218956711,0.000569146591942,-0.000683304573599,0.000386893416529,-0.000779482741275,-0.000846908653256,0.0018538888927,0.00113338374197])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X916b15_X926.ms', tsystable = 'uid___A002_X916b15_X926.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X916b15_X926.ms',
    field = '0',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X916b15_X926.ms.tsys', 'uid___A002_X916b15_X926.ms.wvr.smooth', 'uid___A002_X916b15_X926.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X916b15_X926.ms',
    field = '1',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X916b15_X926.ms.tsys', 'uid___A002_X916b15_X926.ms.wvr.smooth', 'uid___A002_X916b15_X926.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X916b15_X926.ms',
    field = '2',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X916b15_X926.ms.tsys', 'uid___A002_X916b15_X926.ms.wvr.smooth', 'uid___A002_X916b15_X926.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X916b15_X926.ms',
    field = '3',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X916b15_X926.ms.tsys', 'uid___A002_X916b15_X926.ms.wvr.smooth', 'uid___A002_X916b15_X926.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X916b15_X926.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.split') 
  split(vis = 'uid___A002_X916b15_X926.ms',
    outputvis = 'uid___A002_X916b15_X926.ms.split',
    datacolumn = 'corrected',
    spw = '11,13,15,17,19,21,23',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.split.listobs')
  listobs(vis = 'uid___A002_X916b15_X926.ms.split',
    listfile = 'uid___A002_X916b15_X926.ms.split.listobs')
  
  tb.open('uid___A002_X916b15_X926.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X916b15_X926.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X916b15_X926.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X916b15_X926.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X916b15_X926.ms.split',
    mode = 'manual',
    spw = '0:0~7;120~127,1:0~7;120~127,2:0~7;120~127',
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # au.getALMAFluxForMS('uid___A002_X916b15_X926.ms.split',field='1',frequency='88.1841859759GHz')
  setjy('uid___A002_X916b15_X926.ms.split',
      standard='manual', field='1', spix=-0.269071,
      reffreq='88.184186GHz', fluxdensity=[1.868769,0,0,0])

"""
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '0', # center frequency of spw = 88.1841859759GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '1', # center frequency of spw = 98.2261859759GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '2', # center frequency of spw = 100.184185976GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '3', # center frequency of spw = 86.8239948138GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '4', # center frequency of spw = 86.2205097064GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '5', # center frequency of spw = 85.6177265033GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '1', # source name = J0238+1636
    spw = '6', # center frequency of spw = 85.7364398822GHz
    standard = 'manual',
    fluxdensity = [1.85497572169, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
"""  
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X916b15_X926.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.ap_pre_bandpass',
    field = '0', # J0241-0815
    spw = '0:51~76,1:51~76,2:51~76,3:384~576,4:384~576,5:384~576,6:384~576',
    solint = 'int',
    refant = 'DA51',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.split.ap_pre_bandpass', msName='uid___A002_X916b15_X926.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.bandpass',
    field = '', # J0241-0815
    intent = '*BANDPASS*',
    solint = 'inf,0.85449MHz',
    combine = 'scan',
    refant = 'DA51',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X916b15_X926.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.split.bandpass', msName='uid___A002_X916b15_X926.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X916b15_X926.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.phase_int',
    field = '0~2', # J0241-0815,J0238+1636,J0215-0222
    solint = 'int',
    refant = 'DA51',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X916b15_X926.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.split.phase_int', msName='uid___A002_X916b15_X926.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.ampli_inf',
    field = '0~2', # J0241-0815,J0238+1636,J0215-0222
    solint = 'inf',
    refant = 'DA51',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_X916b15_X926.ms.split.bandpass', 'uid___A002_X916b15_X926.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.split.ampli_inf', msName='uid___A002_X916b15_X926.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X916b15_X926.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X916b15_X926.ms.split.flux_inf',
    reference = '1') # J0238+1636
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X916b15_X926.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X916b15_X926.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X916b15_X926.ms.split',
    caltable = 'uid___A002_X916b15_X926.ms.split.phase_inf',
    field = '0~2', # J0241-0815,J0238+1636,J0215-0222
    solint = 'inf',
    refant = 'DA51',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X916b15_X926.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X916b15_X926.ms.split.phase_inf', msName='uid___A002_X916b15_X926.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X916b15_X926.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J0241-0815,J0238+1636
    applycal(vis = 'uid___A002_X916b15_X926.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X916b15_X926.ms.split.bandpass', 'uid___A002_X916b15_X926.ms.split.phase_int', 'uid___A002_X916b15_X926.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X916b15_X926.ms.split',
    field = '2,3', # mira
    gaintable = ['uid___A002_X916b15_X926.ms.split.bandpass', 'uid___A002_X916b15_X926.ms.split.phase_inf', 'uid___A002_X916b15_X926.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J0215-0222
    interp = 'linear,linear',
    calwt = F,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X916b15_X926.ms.split.cal') 
  split(vis = 'uid___A002_X916b15_X926.ms.split',
    outputvis = 'uid___A002_X916b15_X926.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = T)
  
  # Split science target data. We perform limited channel averaging
  # of continuum spws to reduce file size, without losing any information.
  os.system('rm -rf uid___A002_X916b15_X926.ms.split.cal.science') 
  split(vis = 'uid___A002_X916b15_X926.ms.split.cal',
    outputvis = 'uid___A002_X916b15_X926.ms.split.cal.science',
    datacolumn = 'data',
    field='3', spw='0,1,2,3,4,5,6', width=[16,16,16,1,1,1,1],
    keepflags = T)
  
  

